// error.c

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>

#include "error.h"

#ifndef CAT
    #define _CAT(reg, p)  reg ## p
    #define CAT(reg, p)   _CAT(reg, p)
#endif // CAT

static inline void _clearErrorLED(void) {
    CAT(PORT, ERROR_LED_PORT) &=  ~(1 << ERROR_LED_BIT);
}

static inline void _toggleErrorLED(void) {
    CAT(PIN, ERROR_LED_PORT) |=  (1 << ERROR_LED_BIT);
}

void delay_10ms(uint16_t ticks) {
    uint32_t t = F_CPU / 400 * ticks;
    for (ticks = t >> 16; ticks > 0; ticks--)
        _delay_loop_2(0);
    if (t & 0xffff)
        _delay_loop_2(t & 0xffff);
}

static void display_byte(uint8_t b) {
    uint8_t i;
    for (i = b > 0x0f ? 0x80 : 0x08; i > 0; i >>= 1) {
        _toggleErrorLED();
        delay_10ms((i & b) ? 50 : 5);
        _toggleErrorLED();
        delay_10ms(50 + ((i & b) ? 5 : 50));
        if (b == 0x10)
            delay_10ms(25);
    }
    delay_10ms(80);
}


/*
inline static void initErrorLED() {
    CAT(DDR, ERROR_LED_PORT) |= (1 << ERROR_LED_BIT);
}

inline static void setErrorLED(uint8_t st) {
    if (st)
        CAT(PORT, ERROR_LED_PORT) |=  (1 << ERROR_LED_BIT);
    else
        CAT(PORT, ERROR_LED_PORT) &= ~(1 << ERROR_LED_BIT);
}
*/

void toggleErrorLED(void) {
    _toggleErrorLED();
}

inline static void _flashErrorLED() {
    _toggleErrorLED();
    _delay_ms(25);
    _toggleErrorLED();
    _delay_ms(75);
}

void flashErrorLED() {
    int8_t i;
    for (i = 0; i < 5; i++)
        _flashErrorLED();
}

static void sendMsg(uint8_t moduleID, uint16_t errno) {
    _clearErrorLED();
    flashErrorLED();

    delay_10ms(100);                // 1 s off
    display_byte(moduleID);         // display module ID
    if (errno > 255)
        display_byte(errno >> 8);   // display high byte of errno if != 0
    display_byte(errno & 0xff);     // display low byte of errno
    delay_10ms(100);                // 1000 ms off
}

void ledMessage(uint8_t moduleID, uint16_t errno) {
    uint8_t i;
    for (i = 0; i < LEDMESSAGE_REPETITIONS; i++)
        sendMsg(moduleID, errno);
}

void error(uint8_t moduleID, uint16_t errno) {
    cli();

    #ifdef ERROR_ShutdownHook
        ERROR_ShutdownHook;
    #endif // ERROR_ShutdownHook

    while (1)
        sendMsg(moduleID, errno);
}
